require 'sinatra'
require 'sinatra/activerecord'


configure :development do
  # set :database_extras, {:pool => 5, :timeout => 500}
  set :database, 'sqlite3:///' + File.dirname(__FILE__) + '/../delinquo-dev.db'
end

configure :test do
  # set :database_extras, {:pool => 5, :timeout => 500}
  set :database, 'sqlite3:///' + File.dirname(__FILE__) + '/../delinquo-test.db'
end

configure :production do
  # set :database_extras, {:pool => 5, :timeout => 500}
  set :database, 'sqlite3:///' + File.dirname(__FILE__) + '/../delinquo-production.db'
end

require_relative '../models/project'
require_relative '../models/fail'
require_relative '../models/run'

module Delinquo

  class App < Sinatra::Base

    after { ActiveRecord::Base.connection.close }

    # TODO: set public_folder properly
    # set :static, true
    # set :public_folder, File.expand_path('..', __FILE__) + '/public'

    get '/' do
      send_file  File.expand_path('..', __FILE__) + '/public/index.html'
    end

    get '/env' do
      "Env is set to " + settings.environment.to_s
    end

    get '/projects' do
      content_type :json
      projects = Project.all.pluck(:name)
      projects.to_json
    end

    get '/project/:name' do | name |
      content_type :json
      project = Project.where(name: name)
      if project.exists?
        halt 200, project.first.runs.to_json(except: [:id, :project_id], include: { fails: { except: [:id, :run_id]}})
      end
      halt 404, {'Warning' => "project not found with the name #{name}"}, ""
    end

    post '/project/:name' do | name |
      if create_project name 
        halt 201
      end
      halt 409, {'Warning' => "project already exists with the name #{name}"}, ""
    end

    post '/project/:name/run' do | name |
      json = JSON.parse(request.body.read)
      total_tests = json['total']
      fails = json['fails']
      project = Project.where(name: name)
      if project.exists?
        if project.first.runs.count == 10
          project.first.runs.first.delete
        end
        begin
          run = project.first.runs.create! total: total_tests
          fails.each do | fail |
            run.fails.create! fail
          end
          halt 201
        rescue
          halt 400
        end
      end
      halt 404, {'Warning' => "project not found with the name #{name}"}, ""
    end

    get '/report/:name' do | name |
      @name = params[:name].capitalize
      project = Project.where(name: name)
      if project.exists?
        if project.first.runs.count == 0
          halt 200, (erb :no_runs)
        end
        @forecast = project.first.forecast
        @top_fail = project.first.top_fail
        @runs = project.first.runs 
        @runs ||= []
        halt 200, (erb :index)
      end
      halt 200, (erb :not_a_project)
    end

    def create_project(name)
      unless Project.where(name: name).exists?
        if Project.new(name: name).save!
          return true
        end
      end
      false
    end
  end

end
