class Run < ActiveRecord::Base
  belongs_to :project
  has_many :fails
  validates :total, presence: true

  def total_number_of_tests
    total.to_i
  end

  def total_failed
    fails.count
  end

  def total_passed
    (total_number_of_tests - total_failed)
  end

  def percentage_failed
    if total_number_of_tests > 0
    ((total_failed.to_f / total_number_of_tests.to_f) * 100).ceil
    else
      0
    end
  end

  def percentage_passed
    100 - percentage_failed
  end

  def date_time
    format = "%d/%m/%Y %H.%M"
    Time.at(created_at).to_datetime.strftime(format)
  end

  def unique_failing_lines
    collated_failing_lines.uniq
  end

  def collated_failing_lines
    lines = []
    fails.each do |fail|
       lines << fail.line
    end
    lines
  end

end